<?php

// This file created to minimize classes load time. File create time: 2011-03-22 20:27:09



/**
 * @author Max Bond
 * 
 * Q2w3 Table action abstract class
 *
 */
abstract class _q2w3_table_action {
	
	protected $plugin_id;
	
	protected $wp_nonce;
	
	protected $action_page;
	
		
	
	/**
	 * @param string $plugin_id
	 * @param string $action_page GET or POST handler url
	 */
	public function __construct($plugin_id, $action_page = false) {

		$this->plugin_id = $plugin_id;
		
		$this->wp_nonce = wp_create_nonce('q2w3_table_get'); // create nonce for get actions
		
		$this->action_page = $action_page;
		
	}
	
	/**
	 * Returns action name (object class name)
	 * 
	 * @return string
	 */
	public function action_name() {
		
		return get_class($this);
		
	}
	
	/**
	 * Describes action control elements
	 * 
	 * @param object $object instance of _q2w3_table_obj class
	 * @param array $row_data
	 */
	abstract public function html($object = false, $row_data = false);
	
	/**
	 * Place here form data processing code
	 * 
	 * @param string $plugin_id
	 * @param string $object_name
	 */
	public static function action($plugin_id, $object_name) {
		
		 
		
	}
	
}



/**
 * @author Max Bond
 * 
 * Activate selected includes action class
 *
 */
class q2w3_table_activate_selected extends _q2w3_table_action {
	
	
	
	public function html($object = false, $row_data = false) {
		
		return array($this->action_name(), __('Activate', $this->plugin_id));
		
	}
	
	public static function action($plugin_id, $object_name) {
		
		$checked = $_POST['checked'];
	
		if (is_array($checked)) {
			
			$object = new q2w3_include_obj($plugin_id);
	
			foreach ($checked as $id=>$no_value) {
			
				$object->id->val = $id;
		
				$object->change_status(q2w3_include_obj::STATUS_ACTIVE);
					
			}
				
		}
		
	}
	
}



/**
 * @author Max Bond
 * 
 * Change include status action class
 *
 */
class q2w3_table_change_status extends _q2w3_table_action {
	

	
	public function html($object = false, $row_data = false) {
		
		if ($row_data[$object->status->col_name] == q2w3_include_obj::STATUS_ACTIVE) {
		
			$status_change = '<span class="0"><a href="'. $this->action_page .'?action='. $this->action_name() .'&amp;set='. q2w3_include_obj::STATUS_DISABLED .'&amp;object='. get_class($object) .'&amp;id='. $object->id->val .'&amp;wp_nonce='. $this->wp_nonce .'" >'. __('Deactivate', $this->plugin_id) .'</a>'.PHP_EOL;
		
		} elseif ($row_data[$object->status->col_name] == q2w3_include_obj::STATUS_DISABLED) {
		
			$status_change = '<span class="0"><a href="'. $this->action_page .'?action='. $this->action_name() .'&amp;set='. q2w3_include_obj::STATUS_ACTIVE .'&amp;object='. get_class($object) .'&amp;id='. $object->id->val .'&amp;wp_nonce='. $this->wp_nonce .'" >'. __('Activate', $this->plugin_id) .'</a>'.PHP_EOL;
		
		}
		
		return $status_change;
				
	}
	
	public static function action($plugin_id, $object_name) {

		$id = $_GET['id'];
		
		$set = $_GET['set'];
		
		if ($id && $set) {
		
			$object = new $object_name($plugin_id);
		
			$object->id->val = $id;
		
			$object->change_status($set);
		
		}
		
	}
	
}



/**
 * @author Max Bond
 *
 * Delete table record action class
 *
 */
class q2w3_table_delete_row extends _q2w3_table_action {
	
	
	
	public function html($object = false, $row_data = false, $last_record = false) {
				
		if ($last_record) $last_record = '&amp;last_record=1';
		
		return '<span class="0"><a href="'. $this->action_page .'?action='. $this->action_name() .'&amp;object='. get_class($object) .'&amp;id='. $object->id->val.$last_record .'&amp;wp_nonce='. $this->wp_nonce .'" class="delete_row">'. __('Delete', $this->plugin_id) .'</a></span>'.PHP_EOL;
		
	}

	public static function action($plugin_id, $object_name) {
		
		$id = intval($_GET['id']);
		
		if ($id) {
		
			$object = new $object_name($plugin_id);
		
			$object->id->val = $id;
		
			if ($object->delete()) {
				
				q2w3_table_func::change_ref_after_del($object->id->col_name, $id);
						
			}
		
		}
		
	}
	
}



/**
 * @author Max Bond
 * 
 * Delete selected table records action class
 *
 */
class q2w3_table_delete_selected extends _q2w3_table_action {
	
	
	
	public function html($object = false, $row_data = false) {
		
		return array($this->action_name(), __('Delete', $this->plugin_id));
		
	}
	
	public static function action($plugin_id, $object_name) {
		
		$checked = $_POST['checked'];
	
		if (is_array($checked)) {
			
			$object = new $object_name($plugin_id);
				
			foreach ($checked as $id=>$no_value) {
			
				$object->id->val = $id;
		
				if ($object->delete()) {
					
					q2w3_table_func::change_ref_after_del($object->id->col_name, $id);
									
				}
		
			}
				
		}
		
	}
	
}



/**
 * @author Max Bond
 * 
 * Disable selected includes action class
 *
 */
class q2w3_table_disable_selected extends _q2w3_table_action {
	
	
	
	public function html($object = false, $row_data = false) {
		
		return array($this->action_name(), __('Deactivate', $this->plugin_id));
		
	}
	
	public static function action($plugin_id, $object_name) {
		
		$checked = $_POST['checked'];
	
		if (is_array($checked)) {
			
			$object = new q2w3_include_obj($plugin_id);
	
			foreach ($checked as $id=>$no_value) {
			
				$object->id->val = $id;
		
				$object->change_status(q2w3_include_obj::STATUS_DISABLED);
					
			}
				
		}
		
	}
	
}



/**
 * @author Max Bond
 * 
 * Create new table record action class
 *
 */
class q2w3_table_new_row extends _q2w3_table_action {
	

	
	public function html($object = false, $row_data = false) {
		
		return new q2w3_hidden_input(array('name'=>'action', 'value'=>$this->action_name()));
		
	}
	
	public static function action($plugin_id, $object_name) {
		
		$object = new $object_name($plugin_id);
		
		$object->load_values_from_array($_POST['propertie'], 'php2db');
		
		$insert_id = $object->save();
	
		if ($insert_id) {
			
			q2w3_table_func::change_referer(q2w3_table::NEW_MARKER, $insert_id);
					
		}		
		
	}
	
}



/**
 * @author Max Bond
 * 
 * Update table record action class
 *
 */
class q2w3_table_search extends _q2w3_table_action {
	

	
	public function html($object = false, $row_data = false) {
		
		return new q2w3_hidden_input(array('name'=>'action', 'value'=>$this->action_name()));
		
	}
	
	public static function action($plugin_id, $object_name) {
		
		session_start();
		
		$_SESSION['search'] = self::clear($_POST['propertie']);
				
	}
	
	protected static function clear($array) {
		
		foreach ($array as $index=>$value) {
			
			if (is_array($value)) {
				
				$sub_array = self::clear($value);
				
				if ($sub_array) $array[$index] = $sub_array; else unset($array[$index]); 
				
			} elseif (trim($value) == '') {
				
				unset($array[$index]);
				
			}
			
		}
		
		return $array;
		
	}
	
}



/**
 * @author Max Bond
 * 
 * Update table record action class
 *
 */
class q2w3_table_update_row extends _q2w3_table_action {
	

	
	public function html($object = false, $row_data = false) {
		
		return new q2w3_hidden_input(array('name'=>'action', 'value'=>$this->action_name()));
		
	}
	
	public static function action($plugin_id, $object_name) {
		
		$object = new $object_name($plugin_id);
		
		$object->load_values_from_array($_POST['propertie'], 'php2db');
		
		$object->save();
				
	}
	
}



class q2w3_table_wp_page_select extends _q2w3_table_action {
	
	
		
	public function html($object = false, $row_data = false) {
		
		
		
	}	
	
	public static function action($plugin_id, $object_name) {
		
		$_SERVER['HTTP_REFERER'] = '';
		
		$id = $_GET['id'];
		
		$type = $_GET['type'];
		
		$select = new q2w3_select_input();
		
		$select->multiple = true;
		
		$select->style = 'width: 635px; height: 450px;';
		
		$select->id = 'select_wp_pages';
		
		$select->value_to_select = self::selectable_pages($plugin_id, $type);
		
		$select->prepare();
		
		$submit = new q2w3_submit_input(array('value'=>__('Apply', $plugin_id), 'class'=>'button-secondary'));
		
		$cancel = new q2w3_button_input(array('value'=>__('Cancel', $plugin_id), 'class'=>'button-secondary'));
		
		$cancel->js = 'onclick="tb_remove()"';
		
		$res = '<form action="" method="get" id="select_wp_pages_form" style="overflow: hidden">'.PHP_EOL;

		$res .= $select;
		
		$res .= '<div style="color: rgb(33,117,155); float: right; margin-top: 3px">'. __('To select multiple pages or deselect page hold CTRL key', $plugin_id) .'</div>';
		
		$res .= $submit.$cancel;
		
		$res .= '</form>';
		
		$res .= '<script type="text/javascript">';
		
		$res .= 'var selected_values_str = jQuery("#'.$id.' input").attr("value");
				var selected_values_array = selected_values_str.split(",");
				jQuery("#select_wp_pages option").each(function () {
					var opt_val = jQuery(this).attr("value");
					if (jQuery.inArray(opt_val, selected_values_array) > -1) jQuery(this).attr("selected", "true");	
				});';
		
		$res .= 'jQuery("#select_wp_pages_form").submit(function(){
					var str = "";
					var values = "";
					var comma = ",";
					var slash = " // ";
					var i = 0;
					var num = jQuery("#select_wp_pages option:selected").length;
					jQuery("#select_wp_pages option:selected").each(function () {
						i++;
						if (i == num) {
							comma = "";
							slash = "";
						}
						str += jQuery(this).text() + slash;
						values += jQuery(this).attr("value") + comma;					
					});
					jQuery("#'.$id.' input").attr("value", values);
					jQuery("#'.$id.' .selected_pages").text("'. __('Pages', $plugin_id) .': " + str);
					tb_remove();
					return false;
				})';
		
		$res .= '</script>';
				
		echo $res;  
		
	}
	
	/**
 	 * Returns array of selectable pages 
 	 * 
 	 */
	protected static function selectable_pages($plugin_id, $type) { // $type = [include] [exclude]
		
		global $wp_post_types, $wp_taxonomies;
		
		$values_to_select = q2w3_include_obj::page_selectors($plugin_id);
		
		if ($type == 'exclude') array_shift($values_to_select);
		
		$options = get_option(q2w3_inc_manager::ID);	
		
		if (!$options) $options = q2w3_inc_manager::$default_options;
		
		foreach ($options['post_types'] as $post_type=>$params) {
			
			$pages = NULL;
			
			if ($params['enable'] == 'on') {
				
				$post_type_name = $wp_post_types[$post_type]->labels->name;
				
				$selectors = array($post_type.'_all' => __('All', $plugin_id).' '.$post_type_name);
				
				if ($params['expand'] == 'on') {
					
					$pages = q2w3_table_func::select_post_type($post_type);
					
				}
				
				if (!empty($pages)) $selectors = array_merge($selectors, $pages);
				
				$values_to_select[$post_type_name] = $selectors;
				
			}
			
		}
		
		foreach ($options['taxonomies'] as $taxonomy=>$params) {
			
			$pages = NULL;
			
			if ($params['enable'] == 'on') {
				
				$taxonomy_name = $wp_taxonomies[$taxonomy]->labels->name;
				
				$selectors = array($taxonomy.'_all' => __('All', $plugin_id).' '.$taxonomy_name);
				
				if ($params['expand'] == 'on') {
					
					$pages = q2w3_table_func::select_taxonomy($taxonomy);
					
				}
				
				if (!empty($pages)) $selectors = array_merge($selectors, $pages);
				
				$values_to_select[$taxonomy_name] = $selectors;
				
			}
			
		}
		
		/*$pages = q2w3_table_func::wp_pages();
		
		if (!empty($pages)) $values_to_select[__('Pages', $plugin_id)] = $pages;
		
		$pages = q2w3_table_func::wp_categories();
		
		if (!empty($pages)) $values_to_select[__('Categories', $plugin_id)] = $pages;
		
		$pages = q2w3_table_func::wp_tags();
				
		if (!empty($pages)) $values_to_select[__('Tags', $plugin_id)] = $pages;
		
		$pages = q2w3_table_func::wp_posts();
		
		if (!empty($pages)) $values_to_select[__('Posts', $plugin_id)] = $pages;*/

		return $values_to_select;
	
	}
			
}



/**
 * @author Max Bond
 * 
 * Describes basic method for table filters
 *
 */
abstract class _q2w3_table_filter {
	
	protected $plugin_id;
	

	
	public function __construct($plugin_id) {
		
		$this->plugin_id = $plugin_id;
		
	}
	
	abstract public function controls(); // filter controls // html
	
	abstract public function sql(); // filter action // sql 
	
}



/**
 * @author Max Bond
 * 
 * Includes table location filter class
 *
 */
class q2w3_table_location_filter extends _q2w3_table_filter {
	
	protected $inc_obj;
	
	protected $filters;
	
	const VAR_NAME = 'location';
		

	
	public function controls() {
		
		global $wpdb;
		
		$inc_obj = new q2w3_include_obj($this->plugin_id);
		
		// count number of sub filters
		
		$count_filters = $wpdb->get_results('SELECT count(DISTINCT '. $inc_obj->location->col_name .') FROM '. $inc_obj->table(), ARRAY_N); 
		
		if ($count_filters[0][0]) $count_filters = $count_filters[0][0]; else  return false; // if no data to be filtered return false
		
		// Add additional 'Select All' sub filter
				
		$filters['all'] = __('All', $this->plugin_id);
		
		$filters += $inc_obj->location->input_values;
		
		$count_filters++;
		
		$i = 0;
		
		$res = '';
		
		if (key_exists(self::VAR_NAME, $_GET)) $cur_filter = $_GET[self::VAR_NAME]; else $cur_filter = false;
		
		foreach ($filters as $filter_key=>$filter_name) { // sub filter cycle
		
			$condition = 'WHERE '. $inc_obj->location->col_name .' = '. $filter_key;
			
			if ($filter_key == 'all') $condition = false;
			
			// total records for sub filter
			
			$count = $wpdb->get_results('SELECT count(*) FROM '. $inc_obj->table() .' '.$condition, ARRAY_N);
		
			if ($count[0][0] > 0) {
				
				$i++;
				
				if ($i < $count_filters) $separator = '|'; else $separator = '';
												
				$link = q2w3_table_func::change_qstring(self::VAR_NAME, $filter_key); // change filter id
				
				$link = $_SERVER['PHP_SELF'].'?'.q2w3_table_func::change_qstring(q2w3_table_page_filter::VAR_NAME, '', $link); // reset page to number 1
				
				if ($cur_filter == $filter_key || (!$cur_filter && $filter_key == 'all')) $selected = ' class="current"'; else $selected = false;
				
				$res .= '<div style="float: left"><a href="'. $link .'"'. $selected .'>'. $filter_name .' <span class="count">('. $count[0][0] .')</span></a>'. $separator .'</div>'.PHP_EOL;
			
			}
			
		}
		
		if ($res) {
			
			$ul = '<div class="subsubsub">'.PHP_EOL;
			
			$ul .= $res;
			
			$ul .= '</div>'.PHP_EOL;
			
			$this->inc_obj = $inc_obj;
			
			$this->filters = $filters;
			
		}
		
		return $ul;
		
	}
	
	public function sql() {
		
		if (key_exists(self::VAR_NAME, $_GET)) $cur_filter = $_GET[self::VAR_NAME]; else $cur_filter = false;
		
		if (!$cur_filter || $cur_filter == 'all') return false;
		
		if (!key_exists($cur_filter, $this->filters)) return false;
		
		return $this->inc_obj->location->col_name .' = '.$cur_filter;
		
	}
	
	
}



/**
 * @author Max Bond
 * 
 * Table pages support. WP style
 * 
 */
class q2w3_table_page_filter {
	
	protected $plugin_id;
	
	protected $cur_page;
	
	const VAR_NAME = 'cur_page';
		

	
	public function __construct($plugin_id) {
		
		$this->plugin_id = $plugin_id;
		
	}
	
	/**
	 * @return number Current page number
	 */
	public function cur_page() {
		
		if (!$this->cur_page) {
		
			if (key_exists(self::VAR_NAME, $_GET) && intval($_GET[self::VAR_NAME])) $this->cur_page = $_GET[self::VAR_NAME]; else $this->cur_page = 1;
		
		}
		
		return $this->cur_page;
		
	}
	
	/**
	 * @param integer $total_rows Table total rows
	 * @param integer $rows_per_page Table rows per page value
	 * @return string Buttons html
	 */
	public function controls($total_rows, $rows_per_page) {
		
		$total_pages = ceil($total_rows / $rows_per_page); // Count total pages

		if ($total_pages <= 1) return false; // No buttons if there is only one page
		
		$cur_page = $this->cur_page();
		
		if ($cur_page > 1) $prev_page = $cur_page - 1; else $prev_page = false; // Previos page number

		if ($cur_page < $total_pages) $next_page = $cur_page + 1; else $next_page = false; // Next page number
						
		$res = '<div class="tablenav-pages">';
		
		$page_begin = $cur_page*$rows_per_page - $rows_per_page + 1; // Row number of fist row on a page
		
		$page_end = $cur_page*$rows_per_page; // Row number of last row on a page
		
		if ($page_end > $total_rows) $page_end = $total_rows; // Corrects $page_end value for last page
		
		$res .= '<span class="displaying-num">'. $page_begin .'&#8211;'. $page_end .' '. __('from', $this->plugin_id) .' '. $total_rows .'</span>';
		
		if ($prev_page) { // left arrow button
			
			$res .= '<a class="prev page-numbers" href="?'. q2w3_table_func::change_qstring(self::VAR_NAME, $prev_page) .'">&laquo;</a> ';
		
		}
		
		$printed = false;
		
		$printed2 = false;
				
		for ($page = 1; $page <= $total_pages; $page++) { // page cycle
					
			if ($page == 1) { // first page
				
				$res .= $this->page_button($page, $cur_page, self::VAR_NAME);
				
			} elseif ($cur_page - $page >= 3 && !$printed) { // hide unnecessary pages with dots
				
				$res .= '<span class="page-numbers dots">...</span> ';
				
				$printed = true;
				
			}
			
			if (abs($cur_page - $page) <= 2 && $page != $total_pages && $page != 1) { // display two nearest pages
			
				$res .= $this->page_button($page, $cur_page, self::VAR_NAME);
							
			}
			
			if ($page == $total_pages) { // last page
				
				$res .= $this->page_button($page, $cur_page, self::VAR_NAME);
				
			} elseif ($cur_page - $page <= -3 && !$printed2) { // hide unnecessary pages with dots
					
				$res .= '<span class="page-numbers dots">...</span> ';
				
				$printed2 = true;
				
			}	
					
		}
		
		if ($next_page) { // right arrow button
			
			$res .= '<a class="next page-numbers" href="?'. q2w3_table_func::change_qstring(self::VAR_NAME, $next_page) .'">&raquo;</a> ';

		}
		
		$res .= '</div>';
				
		return $res;
				
	}
	
	/**
	 * @param integer $page Current page number in a cycle
	 * @param integer $cur_page Current page selected by user
	 * @param string $var_name $_GET variable name
	 * @return string Button html
	 */
	protected function page_button($page, $cur_page, $var_name) {
			
		if ($page == $cur_page) {
				
			$res = '<span class="page-numbers current">'. $page .'</span> ';
				
		} else {
				
			$res = '<a class="page-numbers" href="?'. q2w3_table_func::change_qstring($var_name, $page) .'">'. $page .'</a> ';
						
		}
			
		return $res;
			
	}
		
}



/**
 * @author Max Bond
 * 
 * Includes table location filter class
 *
 */
class q2w3_table_search_filter extends _q2w3_table_filter {
	
	protected $obj;
	

	
	public function __construct($plugin_id, $obj) {
		
		if ($obj instanceof _q2w3_table_obj) $this->obj = $obj;
		
		session_start();
		
		parent::__construct($plugin_id);
						
	}
	
	public function controls() {
		
		$res = '<div id="poststuff" class="metabox-holder">'.PHP_EOL;
			
		$res .= '<div id="q2w3_table_search" class="postbox" style="width: 100%; margin-bottom: 0px;">'.PHP_EOL;
			
		$res .= '<h3 class="hndle" style="cursor: normal;"><span>Поиск</span></h3>'.PHP_EOL;
			
		if (!empty($_SESSION['search'])) $show_form = 'style="display: block"'; else $show_form = 'style="display: none"';
		
		$res .= '<div class="inside" '. $show_form .'>'.PHP_EOL;
		
		$res .= '<form method="post" action="'. q2w3_table_func::folder_url().'/q2w3_post.php' . '" id="q2w3_table_search_form">'.PHP_EOL;
		
		$search = new q2w3_table_search($this->plugin_id);
		
		$res .= $search->html();
		
		$res .= new q2w3_hidden_input(array('name'=>'wp_nonce', 'value'=>wp_create_nonce('q2w3_table_post')));
		
		$res .= '<table>'.PHP_EOL;
		
		foreach ($this->obj as $pname=>$propertie) {
			
			if ($propertie->search == true) {
				
				$propertie->input->value = $_SESSION['search'][$propertie->col_name];
				
				$res .= '<tr>'.PHP_EOL;
				
				$res .= '<td>'. $propertie->name .'</td>'.PHP_EOL;
				
				$res .= '<td>'. $propertie->input .'</td>'.PHP_EOL;
				
				$res .= '</tr>'.PHP_EOL;
				
			}
			
		}
		
		$res .= '<tr>'.PHP_EOL;
				
		$res .= '<td><input type="button" value="Сброс" class="button-secondary"/></td>'.PHP_EOL;
				
		$res .= '<td><input type="submit" value="Поиск" class="button-primary"/></td>'.PHP_EOL;
		
		$res .= '</tr>'.PHP_EOL;
		
		$res .= '</table>'.PHP_EOL;
		
		$res .= '</form>'.PHP_EOL;
		
		$res .= '</div>'.PHP_EOL.'</div>'.PHP_EOL.'</div>'.PHP_EOL; //'</div>'.PHP_EOL;

		$res .= '<script type="text/javascript">jQuery("#q2w3_table_search .hndle").click(function(){jQuery("#q2w3_table_search .inside").toggle()});';
		
		$res .= 'jQuery("#q2w3_table_search_form .button-secondary").click(function(){jQuery(":input","#q2w3_table_search_form").not(":button, :submit, :reset, :hidden").val("").removeAttr("checked").removeAttr("selected");jQuery("#q2w3_table_search_form").submit();});';
		
		$res .= '</script>';
			
		return $res;
		
	}
	
	public function sql() {
		
		if (!empty($_SESSION['search'])) {
		
			return $this->create_query($_SESSION['search']);
		
		} 
		
		return false;
		
	}
	
	protected function create_query($sarray) {
		
		foreach ($sarray as $scol=>$svalue) {
			
			if (is_array($svalue)) {
			
				foreach ($this->obj as $pname=>$propertie) {
					
					if ($propertie->col_name == $scol) {
						
						$svalue = $propertie->conv->php2db_search($svalue);
						
					}
					
				}
				
				$search[] = "$scol LIKE '$svalue'";
				
			} else {
			
				$search[] = "$scol = '$svalue'";
			
			}
			
		}
		
		return implode(' AND ', $search);
		
	}
	
	
}



/**
 * @author Max Bond
 * 
 * Describes basic data type converting functions
 *
 */
abstract class _q2w3_data_conv {

	protected $propertie;
	
	protected $plugin_id;
	
	public $db2text_values;
	


	/**
	 * @param object $propertie instance of _q2w3_table_obj_propertie
	 * @param unknown_type $plugin_id
	 */
	public function __construct($propertie = false, $plugin_id = false, $db2text_values = false) {
	
		if ($propertie) $this->propertie = $propertie;
		
		if ($plugin_id) $this->plugin_id = $plugin_id;
		
		if ($db2text_values) $this->db2text_values = $db2text_values;
	
	}


	/**
	 * Place here code for converting data before saving in to db
	 * If function is not overloaded in child class, no conversion occurs
	 * 
	 */
	public function php2db($data) {
	
		return $data;
	
	}
	
	/**
	 * Place here code for converting data retrived from db. Used for future proccessing in php (unserialize data and etc.)
	 * If function is not overloaded in child class, no conversion occurs
	 * 
	 */
	public function db2php($data) {
	
		return $data;
	
	}
	
	/**
	 * Place here code for converting data retrived from db. Used for future direct output to the screen (converts db data into readable form)
	 * If function is not overloaded in child class, no conversion occurs
	 * 
	 */
	public function db2text($data) {
	
		return $data;
	
	}

} 



/**
 * @author Max Bond
 * 
 * Checkbox data convert class
 *
 */
class q2w3_checkbox_conv extends _q2w3_data_conv {

	
	
	public function php2db($data) {
			
		if(!$data) return 0; else return 1; // if no data passed return 0
	
	}
	
	public function db2text($data) {
	
		if($data) return __('Yes', $this->plugin_id); else return __('No', $this->plugin_id);
		
	}

}



/**
 * @author Max Bond
 * 
 * Special class for converting data for <select> element with multiple="true" option
 *
 */
class q2w3_checkbox_multi_conv extends _q2w3_data_conv {



	public function php2db($data) {
	
		if (is_array($data) && !empty($data)) {
			
			return implode(",", array_keys($data)); // array became a string. array_keys function needed because selected checkbox ids in array keys
		
		} elseif ($data) {
			
			return $data;
		
		} else {
			
			return 0;
			
		}
	
	}
	
	public function db2php($data) {
		
		if (strpos($data, ',') !== false) $data = explode(',', $data); 
		
		if (!empty($data)) return (array)$data; else return false;
	
	}
	
	public function db2text($data) {
	
		if (!$data) return '-';
		
		if (!$this->db2text_values) return 'missing db2text_values';
		
		if (strstr($data, ',')) $ids = explode(",", $data); else $ids = (array)$data;
		
		foreach ($ids as $id) {
			
			$res[] = $this->db2text_values[$id];
						
		}
		
		$res = implode(', ', $res);
		
		return $res;
				
	}
		
}



class q2w3_date_conv extends _q2w3_data_conv {



	public function php2db($data) {
	
		if (is_array($data) && !empty($data)) {
			
			return $data['year'].'-'.$data['month'].'-'.$data['day'];
		
		} elseif ($data) {
			
			return $data;
		
		} else {
			
			return NULL;
			
		}
	
	}
	
	public function db2php($data) {
	
		if ($data && $data != '0000-00-00') {
			
			list($year,$month,$day) = explode("-", $data);
			
			return array('year'=>$year,'month'=>$month,'day'=>$day);  
		
		} else { 
			
			return NULL;
		
		}
	
	}
	
	public function db2text($data) {
	
		if (!$data) {
			
			return '-';
		
		} elseif ($data != '0000-00-00') {
			
			list($year,$month,$day) = explode("-", $data);
			
			return $day.'-'.$month.'-'.$year;
			
		}
								
	}

}



class q2w3_datetime_conv extends _q2w3_data_conv {



	public function php2db($data) {
	
		if (is_array($data) && !empty($data)) {
			
			$time = $data['h'].':'.$data['m'];
			
			if ($data['s']) $time .= ':'.$data['s'];
			
			return $data['year'].'-'.$data['month'].'-'.$data['day'].' '.$time;
		
		} elseif ($data) {
			
			return $data;
		
		} else {
			
			return NULL;
			
		}
	
	}
	
	public function php2db_search($data) {
		
		if (is_array($data) && !empty($data)) {
			
			if (!$data['year']) $data['year'] = '%';

			if (!$data['month']) $data['month'] = '%';
			
			if (!$data['day']) $data['day'] = '%';
			
				elseif (strlen(trim($data['day'])) == 1) $data['day'] = '0'.$data['day'];
			
			if (!$data['h']) $data['h'] = '%';
			
				elseif (strlen(trim($data['h'])) == 1) $data['h'] = '0'.$data['h'];
			
			if (!$data['m']) $data['m'] = '%';
			
				elseif (strlen(trim($data['m'])) == 1) $data['m'] = '0'.$data['m'];
			
			if (!$data['s']) $data['s'] = '%';
			
				elseif (strlen(trim($data['s'])) == 1) $data['s'] = '0'.$data['s'];
			
			return $data['year'].'-'.$data['month'].'-'.$data['day'].' '.$data['h'].':'.$data['m'].':'.$data['s'];
		
		} 
		
		return false;
		
	}
	
	public function db2php($data) {
	
		if ($data && $data != '0000-00-00 00:00:00') {
			
			list($date, $time) = explode(" ", $data);
			
			list($year,$month,$day) = explode("-", $date);
			
			list($h,$m,$s) = explode(":", $time);
			
			return array('date'=>array('year'=>$year,'month'=>$month,'day'=>$day),'time'=>array('h'=>$h,'m'=>$m,'s'=>$s));  
		
		} else { 
			
			return NULL;
		
		}
	
	}
	
	public function db2text($data) {
	
		if (!$data) {
			
			return '-';
		
		} elseif ($data != '0000-00-00 00:00:00') {
			
			list($date, $time) = explode(" ", $data);
			
			list($year,$month,$day) = explode("-", $date);
			
			list($h,$m,$s) = explode(":", $time);
			
			$time = $h.':'.$m;
			
			if ($s != '00') $time .= ':'.$s;
			
			return $day.'-'.$month.'-'.$year.' '.$time;
			
		}
								
	}

}



/**
 * @author Max Bond
 * 
 * Priority field data convert class
 *
 */
class q2w3_priority_conv extends _q2w3_data_conv {

	
	
	public function php2db($data) {
	
		if(!$data) return 100; else return $data;
	
	}
	
	public function db2php($data) {
	
		if(!$data) return ''; else return $data;
	
	}
	
	public function db2text($data) {
		
		if(!$data) return '-'; else return $data;
		
	}

}



/**
 * @author Max Bond
 * 
 * Select field data conver class
 *
 */
class q2w3_select_conv extends _q2w3_data_conv {



	public function db2text($data) {
			
		if (is_array($this->propertie->input_values)) {
	
			$this->flat_input_values_array();
						
			$data = (array) explode(',',$data);
			
			$res = array();
			
			foreach ($data as $db_value) {
					
				if (key_exists($db_value, $this->propertie->input_values)) $res[] = $this->propertie->input_values[$db_value];
					
			}
				
			return implode(' // ', $res);
				
		}
	
	}
	
	/**
	 * This function is for creating 'flat' input values array. Needed if there are sub arrays in $this->propertie->input_values 
	 * 
	 */
	protected function flat_input_values_array() {
		
		foreach ($this->propertie->input_values as $key=>$value) {
			
			if (is_array($value)) {
				
				unset($this->propertie->input_values[$key]);
				
				$this->propertie->input_values += $value;
				
			}
			
		}
		
	}

}



/**
 * @author Max Bond
 * 
 * Special class for converting data for <select> element with multiple="true" option
 *
 */
class q2w3_select_page_conv extends _q2w3_data_conv {



	public function php2db($data) {
	
		if (is_array($data) && !empty($data)) {
			
			return implode(",", $data); // array became a string
		
		} elseif ($data) {
			
			return $data;
		
		} else {
			
			return 0;
			
		}
	
	}
	
	public function db2php($data) {
	
		if (strstr($data, ',')) return explode(",", $data); else return $data;
	
	}
	
	public function db2text($data) {
	
		if (!$data) return '-';
		
		if (strstr($data, ',')) $ids = explode(",", $data); else $ids = (array) $data;
		
		$pages = '';
		
		foreach ($ids as $id) {
			
			$pages .= q2w3_select_page_conv::page_title($id).' // ';
						
		}
		
		$pages = substr_replace($pages, '', -4);
		
		return $pages;
				
	}
	
	
	/**
	 * Return page title
	 * 
	 * @param string $id Page id
	 * @return string Page title or false if title not found
	 */
	public static function page_title($id) {
		
		static $pages = array();
		
		if (!$pages) {
		
			$pages = q2w3_table_func::page_selectors();
		
		}
				
		if (isset($pages[$id]) && !empty($pages[$id])) {
			
			return $pages[$id];
			
		} else {
			
			return false;
			
		}
			
	}
	
}



/**
 * @author Max Bond
 * 
 * Class for input text conversion
 *
 */
class q2w3_text_conv extends _q2w3_data_conv {

	
	
	public function php2db($data) {
	
		if(!$data) return 0; else return $data; 
	
	}
	
	public function db2php($data) {
	
		if(!$data) return ''; else return $data;
	
	}
	
}



class q2w3_time_conv extends _q2w3_data_conv {



	public function php2db($data) {
	
		if (is_array($data) && !empty($data)) {
			
			$time = $data['h'].':'.$data['m'];
			
			if ($data['s']) $time .= ':'.$data['s'];
			
			return $time;
		
		} elseif ($data) {
			
			return $data;
		
		} else {
			
			return NULL;
			
		}
	
	}
	
	public function db2php($data) {
	
		if ($data && $data != '00:00:00') {
			
			list($h,$m,$s) = explode(":", $data);
			
			return array('h'=>$h,'m'=>$m,'s'=>$s);  
		
		} else { 
			
			return NULL;
		
		}
	
	}
	
	public function db2text($data) {
	
		if (!$data) {
			
			return '-';
		
		} elseif ($data != '00:00:00') {
			
			list($h,$m,$s) = explode(":", $data);
			
			$time = $h.':'.$m;
			
			if ($s != '00') $time .= ':'.$s;
			
			return $time;
			
		}
								
	}

}



/**
 * @author Max Bond
 * 
 * Describes basic methods for <input> classes
 *
 */
abstract class _q2w3_input {
	
	public $name; // <input name=""/> attribute value
	
	public $value; // <input value=""/> attribute value // also used for <select> to input value of selected elements
	
	public $value_to_select; // hear you can set values for checkbox, radio elements // for select element values it should be an associative array ('id'=>'name') // for groups use array in array: array('group_name'=>array('id'=>'name'))
	
	public $disabled; // <input disabled=""/> attribute value
	
	public $multiple; // <input multiple=""/> attribute value
	
	public $checked; // <input checked=""/> attribute value
	
	public $maxlength; // maxlenght attribute value
	
	public $class; // element css class name
	
	public $id; // element id
	
	public $style; // element style
	
	public $js; // element javascript code
	
		
	
	function __construct($values = false) {
		
		$this->set($values);
		
	}
	
	/**
	 * Quickly converts class to a string. Used for echo $object 
	 */
	public function __toString() {
				
		$html = $this->html(); 
		
		if (!$html) return ''; else return $html;
		
	}
	
	
	/**
	 * Used for setting object properties with array values. Array keys are propertie names and array values are properties values
	 */
	public function set($values) {
		
		if (is_array($values)) {
			
			foreach ($this as $key=>$value) {
				
				if (key_exists($key, $values)) $this->$key = $values[$key];
				
			}
			
		}
		
		$this->prepare();
		
	}
	
	/**
	 * Cnverts properties values to html form
	 */
	public function prepare() {
		
		if ($this->disabled) $this->disabled = ' disabled="disabled"';
		if ($this->multiple) $this->multiple = ' multiple="multiple"';
		if ($this->checked) $this->checked = ' checked="checked"';
		if ($this->maxlength) $this->maxlength = ' maxlength="'. $this->maxlength .'"';
		if ($this->class) $this->class = ' class="'. $this->class .'"';
		if ($this->id) $this->id = ' id="'. $this->id .'"';
		if ($this->style) $this->style = ' style= "'. $this->style .'"';
		if ($this->js) $this->js = ' '.$this->js;
							
	}
	
	/**
	 * This function must be overloaded in a child class. Used for providing form element html code 
	 */
	abstract public function html();
	
}



/**
 * @author Max Bond
 * 
 * <input type="button"/> element
 *
 */
class q2w3_button_input extends _q2w3_input {
	
	
	
	public function html() {
		
		return '<input type="button" value="'. $this->value .'"'. $this->class.$this->id.$this->style.$this->js .'/>'.PHP_EOL;
		
	}
	
}



/**
 * @author Max Bond
 * 
 * <input type="checkbox"/> element
 *
 */
class q2w3_checkbox_input extends _q2w3_input {
	
	
	
	public function html() {
		
		if (!$this->checked && $this->value == $this->value_to_select) $this->checked = ' checked="checked"';
				
		return '<input type="checkbox" name="'. $this->name .'" value="'. $this->value_to_select .'"'. $this->class.$this->id.$this->checked.$this->disabled.$this->style.$this->js .'/>';
		
	}
	
}



class q2w3_checkbox_multi_input  extends _q2w3_input {
	
	
	
	public function html() {
		
		/*if ($this->value) {
			
			$values = implode(',', (array)$this->value);
								
		}
		
		var_dump($this->value);*/
		
		$roles = $this->value_to_select;
		
		foreach ($roles as $role=>$name) {
			
			if (in_array($role, (array)$this->value)) $checked = true; else $checked = false;
			
			$checkbox = new q2w3_checkbox_input(array('name'=>$this->name.'['.$role.']', 'checked'=>$checked, 'value'=>1));
		
			$res[] =  $checkbox . '&nbsp;'. $name;
		
		}
		
		$res = implode(', ', $res);
		
		return $res;
		
	}
	
}



class q2w3_date_input  extends _q2w3_input {
	
	
	
	public function html() {
		
		$day = new q2w3_text_input(array('name'=>$this->name.'[day]', 'class'=>'day_input', 'maxlength'=>2, 'value'=>$this->value['day']));
		
		$month = new q2w3_month_input(array('name'=>$this->name.'[month]', 'value'=>$this->value['month']));
		
		$year = new q2w3_text_input(array('name'=>$this->name.'[year]', 'class'=>'year_input', 'maxlength'=>4, 'value'=>$this->value['year']));
		
		return $day.'&nbsp;'.$month.'&nbsp;'.$year;
		
	}
	
	
}



class q2w3_datetime_input  extends _q2w3_input {
	
	
	
	public function html() {
		
		$date = new q2w3_date_input(array('name'=>$this->name, 'value'=>$this->value['date']));
		
		$time = new q2w3_time_input(array('name'=>$this->name, 'value'=>$this->value['time']));
		
		return $date.'&nbsp;'.$time;
		
	}
		
}



/**
 * @author Max Bond
 * 
 * <input type="file"/> element
 *
 */
class q2w3_file_input extends _q2w3_input {
	
	
	
	public function html() {
		
		return '<input type="file" name="'. $this->name .'"'. $this->class.$this->id.$this->style.$this->js .'/>'.PHP_EOL;
		
	}
	
}



/**
 * @author Max Bond
 * 
 * <input type="hidden"/> element
 *
 */
class q2w3_hidden_input extends _q2w3_input {
	
	
	
	public function html() {
		
		if ($this->value) {
			
			return '<input type="hidden" name="'. $this->name .'" value="'. $this->value .'"'. $this->class.$this->id .'/>'.PHP_EOL;
		
		} else {
				
			return false;
		
		}
	
	}
		
}



class q2w3_month_input extends _q2w3_input {
	
	public static $months = array(
			''=>'',
			'01'=>'Января',
			'02'=>'Февраля',
			'03'=>'Марта',
			'04'=>'Апреля',
			'05'=>'Майя',
			'06'=>'Июня',
			'07'=>'Июля',
			'08'=>'Августа',
			'09'=>'Сентября',
			'10'=>'Октября',
			'11'=>'Ноября',
			'12'=>'Декабря');
	
	public function html() {
		
		$selector = new q2w3_select_input(array('name'=>$this->name, 'value_to_select'=>self::$months, 'class'=>'month_input', 'value'=>$this->value));
		
		return $selector->html();
		
	}
	
}



/**
 * @author max Bond
 * 
 * <input type="password"/> element
 *
 */
class q2w3_password_input extends _q2w3_input {
	
	
	
	public function html() {
				
		return '<input type="password" name="'. $this->name .'" value="'. $this->value .'"'. $this->class.$this->id.$this->maxlength.$this->style.$this->js .'/>'.PHP_EOL;
		
	}
	
}



/**
 * @author Max Bond
 * 
 * <input type="radio"/> element
 *
 */
class q2w3_radio_input extends _q2w3_input {
	
	
	
	public function html() {
		
		if (!$this->checked && $this->value == $this->value_to_select) $this->checked = ' checked="checked"';
		
		return '<input type="radio" name="'. $this->name .'" value="'. $this->value_to_select .'"'. $this->class.$this->id.$this->checked.$this->disabled.$this->style.$this->js .'/>';
		
	}
	
}



/**
 * @author Max Bond
 * 
 * <select> element
 *
 */
class q2w3_select_input extends _q2w3_input {
	
	
	
	public function html() {
		
		if (!(is_array($this->value_to_select) && !empty($this->value_to_select))) return false; // do nothing if there are no select items
		
		$string = '<select name="'. $this->name .'"'. $this->class.$this->id.$this->multiple.$this->style.$this->js .'>'.PHP_EOL;
				
		foreach ($this->value_to_select as $key=>$name) {
		
			if (is_array($name) && !empty($name)) {
			
				$string .= '<optgroup label="'. $key .'">'.PHP_EOL;
				
				foreach ($name as $opt_key=>$opt_name) {
				
					$string .= $this->make_option($opt_key, $opt_name);
				
				}
				
				$string .= '</optgroup>'.PHP_EOL;
			
			} else {
		
				$string .= $this->make_option($key, $name);
			
			}
			
		}
		
		$string .= '</select>'.PHP_EOL;
		
		return $string;
		
	}
	
	protected function make_option($key, $name) {
	
		if (is_array($this->value)) { // for multiple select
		
			if (in_array($key, $this->value)) $selected = 'selected="selected"'; else $selected = false; 
		
		} elseif ($this->value == $key) { // for single select
		
			$selected = 'selected="selected"';
		
		} else {
		
			$selected = false;
		
		}
						
		return "	<option value=\"$key\" $selected>$name</option>".PHP_EOL;
	
	}
	
}



/**
 * @author Max Bond
 * 
 * <input type="submit"/> element
 *
 */
class q2w3_submit_input extends _q2w3_input {
	
	
	
	public function html() {
		
		return '<input type="submit" value="'. $this->value .'"'. $this->class.$this->id.$this->style.$this->js .'/>'.PHP_EOL;
		
	}
	
}



/**
 * @author Max Bond
 * 
 * <input type="text"/> element
 *
 */
class q2w3_text_input extends _q2w3_input {
	
	
	
	public function html() {
		
		return '<input type="text" name="'. $this->name .'" value="'. $this->value .'"'. $this->class.$this->id.$this->maxlength.$this->style.$this->js .'/>'.PHP_EOL;
		
	}
	
}



/**
 * @author Max Bond
 * 
 * Output text instead of <input type="text"/> element
 *
 */
class q2w3_text_wo_input extends _q2w3_input {
	
	
	
	public function html() {
		
		return '<span '. $this->class.$this->id.$this->style.$this->js .'>'. $this->value .'</span><input type="hidden" name="'. $this->name .'" value="'. $this->value .'"'. $this->class.$this->id .'/>'.PHP_EOL;
		
	}
	
}



/**
 * @author Max Bond
 * 
 * <textarea> element
 *
 */
class q2w3_textarea_input extends _q2w3_input {
	
	
	
	public function html() {
		
		return '<textarea name="'. $this->name .'"'. $this->class.$this->id.$this->style.$this->js .'>'. $this->value .'</textarea>'.PHP_EOL;;
		
	}
	
}



class q2w3_time_input  extends _q2w3_input {
	
	public $seconds = false;
	
	
	
	public function html() {
		
		$h = new q2w3_text_input(array('name'=>$this->name.'[h]', 'class'=>'h_input', 'maxlength'=>2, 'value'=>$this->value['h']));
		
		$m = new q2w3_text_input(array('name'=>$this->name.'[m]', 'class'=>'m_input', 'maxlength'=>2, 'value'=>$this->value['m']));
		
		if ($this->seconds) {
			
			$s = ' : '.$s = new q2w3_text_input(array('name'=>$this->name.'[s]', 'class'=>'s_input', 'maxlength'=>2, 'value'=>$this->value['s']));
		
		}
		
		return $h.' : '.$m.$s;
		
	}
	
	
}



class q2w3_wp_page_select_input  extends _q2w3_input {
	
	public $type;
	
	
	
	public function html() {
		
		if ($this->type == 'include') {
			
			$type = '&amp;type=include';
			
			$field_id = 'inc_pages_select'; 
		
		} elseif ($this->type == 'exclude') {
			
			$type = '&amp;type=exclude';
			
			$field_id = 'exc_pages_select';
			
		} else {
			
			$type = false;
			
			$field_id = false;
			
		}
		
		if ($this->value) {
			
			$value = (array) $this->value;
			
			$saved_values = implode(',', $value);
			
			$pages = '';
			
			foreach ($value as $id) {
				
				$pages .= q2w3_select_page_conv::page_title($id).' // ';
				
			}
			
			$pages = substr_replace($pages, '', -4);
		
		}
		
		$res = '<div '. $this->id .'>'.PHP_EOL;
		
		$res .= '<div><a href="'. site_url() . '/wp-content/plugins/q2w3-inc-manager/q2w3-table/q2w3_get.php?width=640&amp;height=480&amp;action=q2w3_table_wp_page_select&amp;id='. $field_id . $type .'&amp;wp_nonce='. wp_create_nonce('q2w3_table_get') .'" title="'. __('Select pages', q2w3_inc_manager::ID) .'" class="thickbox" style="text-decoration: none">[...]</a></div>'.PHP_EOL;
		
		$res .= '<input type="hidden" name="'. $this->name .'" value="'. $saved_values .'"/>';
		
		$res .= '<div class="selected_pages">'. __('Pages', q2w3_inc_manager::ID) .': '. $pages .'</div>'.PHP_EOL;
		
		$res .= '</div>'.PHP_EOL;
		
		return $res;
		
	}
	
	
}



/**
 * @author Max Bond
 * 
 * Abstract _q2w3_table_obj class. Describes basic methods for all _q2w3_table_class children classes
 *
 */
abstract class _q2w3_table_obj {

	protected static $plugin_id; // Plugin ID  

	
	
	/**
	 * Object init
	 * 
	 * @param string $plugin_id Plugin ID
	 * @param integer $object_id Object ID (db primary key value)
	 * @param string $conv_type 'php2db' | 'db2php' | 'db2text'
	 */
	public function __construct($plugin_id, $object_id = false, $conv_type = false) {
	
		self::$plugin_id = $plugin_id;		
		
		foreach ($this as $pname=>$propertie) {
					
			$this->$pname = new _q2w3_table_obj_propertie(); // create object properties
		
		}
		
		$this->init(); // init of properties
				
		if ($object_id && $conv_type) { // if params specified load object data from db
			
			$this->load_values_from_db($object_id, $conv_type);
		
		} else { // if not check for saved input
			
			$this->get_saved_input();
			
		}
			
	}

	
	
	abstract public function table(); // method for providing db table name
	
	abstract public function table_ver(); // method for providing db table version 
	
	abstract protected function input_name($name); // method for providing value for from <input name=""/> attribute
	
	abstract protected function init(); // object properties init method
	
	
	
	/**
	 * Method for verifing user input. Extend this function in a child class
	 * 
	 */
	protected function check_input_data() {
	
		return true;
	
	}
	
	/**
	 * Method for launching actions after data inserted into db. Extend this function in a child class
	 * 
	 */
	protected function after_insert() {
		
		return false;
		
	}
	
	/**
	 * Method for launching actions after data db data updated. Extend this function in a child class
	 * 
	 */
	protected function after_update() {
		
		return false;
		
	}
	
	/**
	 * Method for launching actions after db data deleted. Extend this function in a child class
	 * 
	 */
	protected function after_delete() {
		
		return false;
		
	}
	
	/**
	 * Method for providing javascript support for properties edit window. Extend this function in a child class
	 * 
	 */
	public function js() {
		
		return false;
		
	}
	
	/**
	 * Saves object data
	 * 
	 */
	public function save() {
	
		global $wpdb; // use wpdp object
						
		if ($this->check_input_data()) { // valid user input
			
			if ($this->id->val && $this->id->val != q2w3_table::NEW_MARKER) { // update
				
				if ($wpdb->update($this->table(), $this->values_array(), array($this->id->col_name=>$this->id->val)) !== false) { // ok	
		
					new q2w3_table_sys_msg(__('Record updated', self::$plugin_id)); // message to user
					
					$this->after_update(); // actions after update
										
					return true;
		
				} else { // error
					
					new q2w3_table_sys_msg(__('DB error:', self::$plugin_id).' '.$wpdb->last_error);
					
					return false;
					
				}	
					
			} elseif ($this->id->val && $this->id->val == q2w3_table::NEW_MARKER) { // insert
								
				$this->id->val = 0; // id value must be 0 when inserting data
				
				if ($wpdb->insert($this->table(), $this->values_array()) !== false) { // ok				
			
					$this->after_insert(); // action after insert
					
					new q2w3_table_sys_msg(__('Record inserted', self::$plugin_id)); // message to user
					
					return $wpdb->insert_id; // return inserted id
					
				} else { // error
					
					new q2w3_table_sys_msg(__('DB error:', self::$plugin_id).' '.$wpdb->last_error);
					
					return false;
					
				}
			
			}
		
		} else { // user input had errors
			
			$this->save_input(); // save data for faster input  
			
			return false;
			
		}
	
	}
	
	/**
	 * Deletes object data from db
	 * 
	 */
	public function delete() {
	
		global $wpdb; // use wpdb object
	
		if ($this->id->col_name && $this->id->val) { // id supplied
	
			if ($wpdb->query('DELETE FROM '. $this->table() .' WHERE '. $this->id->col_name .' = '. $this->id->val) !== false) { // ok
			
				$this->after_delete(); // actions after delete
				
				new q2w3_table_sys_msg(__('Record(s) deleted', self::$plugin_id)); // msg for user
				
				return true;
			
			} else { // error
				
				new q2w3_table_sys_msg(__('DB error:', self::$plugin_id).' '.$wpdb->print_error()); // error msg
				
				return false;
				
			}
			
		} else { // no id 
		
			return false;
		
		}
	
	}
	
	/**
	 * Returns array of object properties values
	 * 
	 */
	public function values_array() {
		
		foreach ($this as $pname=>$propertie) {
			
			$array[$propertie->col_name] = $propertie->val;
			
		}
		
		return $array;
	
	}
	
	/**
	 * Sets object properties values to NULL
	 * 
	 */
	public function clean_values() {
		
		foreach ($this as $pname=>$propertie) {
			
			$this->$pname->val = NULL;
			
		}
		
	}
	
	
	/**
	 * Loads object data from db
	 * 
	 * @param integer $object_id
	 * @param string $conv_type 'php2db' | 'db2php' | 'db2text'
	 */
	public function load_values_from_db($object_id, $conv_type) {
	
		global $wpdb;
	
		if ($object_id) {
			
			$data = $wpdb->get_row('SELECT * FROM '. $this->table() .' WHERE id = '.$object_id, ARRAY_A);
			
			if (!empty($data)) $this->load_values_from_array($data, $conv_type);
		
		}
	
	}
	
	
	/**
	 * Loads object data from array
	 * 
	 * @param array $data
	 * @param string $conv_type 'php2db' | 'db2php' | 'db2text'
	 */
	public function load_values_from_array($data, $conv_type) {
	
		if ($data) {
			
			foreach ($this as $pname=>$propertie) {
				
				if (key_exists($propertie->col_name, $data)) $value = $data[$propertie->col_name]; else $value = 0;
				
				if ($conv_type == 'php2db') { // if data passed for storing in db
						
					if (is_object($propertie->conv)) $this->$pname->val = $propertie->conv->php2db($value); else $this->$pname->val = $value;
						
					$this->prepare_data_db($pname);
					
				} elseif ($conv_type == 'db2php') { // if data passed from db for future proccess by php (also creating html forms)
						
					$this->prepare_data_php($pname, $value);
						
					if (is_object($propertie->conv)) $this->$pname->val = $propertie->conv->db2php($this->$pname->val); 
					
				} elseif ($conv_type == 'db2text') { // if data passed from db for direct screen output (used for table rows)
						
					$this->prepare_data_php($pname, $value);
						
					if (is_object($propertie->conv)) $this->$pname->val = $propertie->conv->db2text($this->$pname->val);
										
				}
							
			}
								
		}
	
	}
	
	/**
	 * Sanitizes propertie values before saving in db
	 * 
	 * @param string $pname Object propertie name
	 */
	protected function prepare_data_db($pname) {
	
		static $charset = '';
		
		if (!$charset) $charset = get_option('blog_charset');
		
		$this->$pname->val = htmlspecialchars(trim($this->$pname->val), ENT_QUOTES, $charset, false);
		
	}
	
	
	/**
	 * Rmoves slashes from propertie value passed from db
	 * 
	 * @param string $pname Object propertie name
	 * @param string $value Object propertie value
	 */
	protected function prepare_data_php($pname, $value) {
	
		$this->$pname->val = stripslashes($value);
			
	}

	/**
	 * Creates object db table 
	 * Uses dbDelta function for table upgrade tasks
	 * 
	 */
	public function create_table() {
	
		global $wpdb;
   
		$table_name = $this->table();
		
		$table_ver = $this->table_ver();
		
		$cur_table_ver = get_option($table_name.'_table_ver');
		
		if ($wpdb->get_var("SHOW TABLES LIKE '$table_name'") != $table_name || $cur_table_ver != $table_ver) { // if table not exists or different table version 
      
			$sql = "CREATE TABLE $table_name (". $this->table_structure() .') TYPE=MyISAM CHARACTER SET utf8 COLLATE utf8_general_ci;'; 
				
			require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
	      
			dbDelta($sql);
	
			if ($cur_table_ver) { // table version option
				
				update_option($table_name.'_table_ver', $table_ver, '', 'no');
				
			} else {
	
				add_option($table_name.'_table_ver', $table_ver, '', 'no');
			
			}
			
						
			if ($cur_table_ver < 1.4) { // Update to db ver 1.4
				
				$sql = "UPDATE $table_name SET hide_from_admin = 'administrator' WHERE hide_from_admin = 1";
				
				$wpdb->query($sql);
				
			}
							
		}
				
	}
	
	
	/**
	 * Returns SQL code to be used in create table method
	 * Code formatted to comply dbDelta function requirements
	 * 
	 * @return string
	 */
	protected function table_structure() {
	
		foreach ($this as $key=>$prop) {
		
			$sql .= $prop->col_name .' '. $prop->col_type .','.PHP_EOL;
			
			if ($prop->col_index == _q2w3_table_obj_propertie::PRIMARY) {
			
				$primaries[] = $prop->col_name;
			
			} elseif ($prop->col_index == _q2w3_table_obj_propertie::UNIQUE) {
			
				$uniques[] = $prop->col_name;
			
			} elseif ($prop->col_index == _q2w3_table_obj_propertie::INDEX) {
			
				$indexes[] = $prop->col_name;
			
			}
		
		}
		
		if (is_array($primaries)) {
		
			$index_keys[0] = 'PRIMARY KEY  (';
		
			$i = 0;
			
			foreach ($primaries as $primary) {
		
				$i++;
		
				$index_keys[0] .= $primary;
		
				if ($i == count($primaries)) $index_keys[0] .= ')'; else $index_keys[0] .= ',';
		
			}
		
		}
		
		if (is_array($uniques)) {
		
			$index_keys[1] = 'UNIQUE KEY '. $uniques[0] .' (';
		
			$i = 0;
		
			foreach ($uniques as $unique) {
		
				$i++;
		
				$index_keys[1] .= $unique;
		
				if ($i == count($uniques)) $index_keys[1] .= ')'; else $index_keys[1] .= ',';
		
			}
		
		}
		
		if (is_array($indexes)) {
		
			$index_keys[2] = 'KEY '. $indexes[0] .' (';
		
			$i = 0;
		
			foreach ($indexes as $index) {
		
				$i++;
		
				$index_keys[2] .= $index;
		
				if ($i == count($indexes)) $index_keys[2] .= ')'; else $index_keys[2] .= ',';
		
			}
			
		}
			
		return PHP_EOL.$sql.implode(','.PHP_EOL, $index_keys).PHP_EOL;
			
	}
	
	/**
	 * Saves user input in a session variable
	 * 
	 */
	protected function save_input() {
		
		$_SESSION['q2w3_table_tmp_data'] = $this->values_array();
		
	}
	
	
	/**
	 * Loads object data from saved input
	 * Performed only once 
	 * 
	 */
	protected function get_saved_input() {
		
		if (key_exists('id', $_GET) && $_GET['id'] == q2w3_table::NEW_MARKER && is_array($_SESSION['q2w3_table_tmp_data']) && !empty($_SESSION['q2w3_table_tmp_data'])) {
		
			$this->load_values_from_array($_SESSION['q2w3_table_tmp_data'], 'db2php');
		
			unset($_SESSION['q2w3_table_tmp_data']);
		
		}
		
	}
	
}



/**
 * @author Max Bond
 * 
 * This class describes properties of _q2w3_table_obj class properties
 *
 */
class _q2w3_table_obj_propertie {

	public $name; // Propertie name // will be displayed to user
	
	public $col_name; // database column name
	
	public $col_type; // database column type
	
	public $col_index; // database column index
	
		const PRIMARY = 1; // ID of PRIMARY KEY index
	
		const UNIQUE = 2; // ID of UNIQUE KEY index
		
		const INDEX = 3; // ID of INDEX index
	
	public $val; // propertie value
	
	public $input_values; // <select> options array is stored here
	
	public $input; // input object // instance of _q2w3_input class
	
	public $conv; // data conver object // instance of _q2w3_data_conv class
	
	public $help; // Propertie help // mat be displayed for user

	public $table_view; // if set to true table column will be displayed
	
	public $table_view_change; // if set to true $table_view propertie can be set by user
	
	public $single_view; // if set to true value input form element will be displayed at the object edit window
	
	public $search; // search this field or not

}



/**
 * @author Max Bond
 * 
 * Describes db table for storing includes and provides methods for includes data manipulation 
 * 
 */
class q2w3_include_obj extends _q2w3_table_obj {
	
	public $id; // Include ID

	public $title; // Include title/description
	
	public $status; // Include status
	
		const STATUS_ACTIVE = '10';
		
		const STATUS_DISABLED = '20';
	
	public $location; // Include location
	
		const LOC_HEADER = '10'; // inside <head></head> section // displays with get_header() function
		
		const LOC_FOOTER = '20'; // bottom of the page // displays with get_footer() function
		
		const LOC_WIDGET = '30'; // sidebar // widget position could be set at widget settings page 
		
		const LOC_B_PAGE_CONTENT = '40'; // before wp loop 
		
		const LOC_A_PAGE_CONTENT = '50'; // after wp loop
		
		const LOC_B_POST_CONTENT = '60'; // before page/post content
		
		const LOC_A_POST_EXCERPT = '65'; // after post excerpt
		
		const LOC_A_POST_CONTENT = '70'; // after page/post content
		
		const LOC_MANUAL = '80'; // manual location
		
		const LOC_SHORTCODE = '90'; // manual location
		
	public $widget_title; // title of the widget // displays only on public pages
	
	public $priority; // prority of the include
	
	public $inc_pages; // list of pages where include will be shown
			
		const ALL = 'all'; // all pages
		
		//const WP_PAGES = 'page_all'; // all WordPress pages (not any generic webpage from your blog)
		
		const POST_TYPES_PAGES = 'post_types_pages'; // all posts pages
		
		const TAX_PAGES = 'taxonomy_all';
		
		const DATE_PAGES = 'date_pages'; // all date-based archive page is being displayed (i.e. a monthly, yearly, daily or time-based archive)
		
		//const CATEGORY_PAGES = 'category_pages'; // When any Category archive page is being displayed
		
		//const TAG_PAGES = 'tag_pages'; // When any Tag archive page is being displayed

		const AUTHOR_PAGES = 'author_pages'; // When any Author page is being displayed 

		const ATTACHMENT_PAGES = 'attachment_pages'; // When an attachment document to a post or Page is being displayed
		
		const PAGED_PAGES = 'paged_pages'; // When the page being displayed is "paged". This refers to an archive or the main page being split up over several pages and will return true on 2nd and subsequent pages of posts. This does not refer to a Post or Page whose content has been divided into pages using the <!--nextpage--> QuickTag
		
		const FRONT_PAGE = 'front_page'; // When it is the front of the site displayed, whether it is posts or a Page
		
		const SEARCH_PAGE = 'search_page'; // When a search result page archive is being displayed
		
		const PREVIEW_PAGE = 'preview_page'; // When a single post being displayed is viewed in Draft mode

		const PAGE_404 = 'page_404'; // When a page displays after an "HTTP 404: Not Found" error occurs
					
	public $exc_pages; // list of pages where include will not be shown
	
	public $hide_from; // option for hiding include from logged in admin
	
	public $code_align; // text-align type of the included code

		const ALIGN_LEFT = '1';
		
		const ALIGN_CENTER = '2';
		
		const ALIGN_RIGHT = '3';
		
	public $code; // included code
	
	
	
	/**
 	 * Returns db table name based on plugin id 
 	 * 
 	 */
	public function table() {
		
		global $wpdb;
	
		return $wpdb->prefix . q2w3_inc_manager::ID;
		
	}
	
	/**
 	 * Returns db table version 
 	 * Needed for upgrading db table with the db_delta function
 	 * 
 	 */
	public function table_ver() {
		
		return '1.4';
		
	}
	
	/**
 	 * Returns name used by <input name="" /> attribute 
 	 * 
 	 */
	protected function input_name($name) {
	
		return "propertie[$name]";
	
	}
	
	/**
 	 * Main object init function
 	 * Here described all db columns types, form elements for data input and display behavior
 	 * Refer to _q2w3_table_obj_properties class for field options description 
 	 * 
 	 */
	protected function init() {
		
		$this->id->name = 'ID';
		$this->id->col_name = 'id';
		$this->id->col_type = 'INT NOT NULL AUTO_INCREMENT';
		$this->id->col_index = _q2w3_table_obj_propertie::PRIMARY;
		$this->id->input = new q2w3_hidden_input(array('name'=>$this->input_name($this->id->col_name)));
		$this->id->table_view = false;
		$this->id->table_view_change = false;
		$this->id->single_view = false;
		
		$this->title->name = __('Description', self::$plugin_id);
		$this->title->col_name = 'description';
		$this->title->col_type = 'VARCHAR(255) NOT NULL';
		$this->title->input = new q2w3_text_input(array('name'=>$this->input_name($this->title->col_name), 'class'=>'text_input', 'maxlength'=>255));
		$this->title->help = __('Name/description of the included code. Required field', self::$plugin_id);
		$this->title->table_view = true;
		$this->title->table_view_change = false;
		$this->title->single_view = true;
				
		$this->status->name = __('Status', self::$plugin_id);
		$this->status->col_name = 'status';
		$this->status->col_type = 'INT NOT NULL';
		$this->status->input_values = array(self::STATUS_ACTIVE=>__('Active', self::$plugin_id), self::STATUS_DISABLED=>__('Disabled', self::$plugin_id));
		$this->status->input = new q2w3_select_input(array('name'=>$this->input_name($this->status->col_name), 'value_to_select'=>$this->status->input_values, 'class'=>'select_input'));
		$this->status->help = __('Active - code will be displayed, Disabled - code will not be displayed', self::$plugin_id);
		$this->status->conv = new q2w3_select_conv($this->status, self::$plugin_id);
		$this->status->table_view = true;
		$this->status->table_view_change = false;
		$this->status->single_view = true;
		
		$this->location->name = __('Location', self::$plugin_id);
		$this->location->col_name = 'location';
		$this->location->col_type = 'INT NOT NULL';
		$this->location->input_values = array(self::LOC_HEADER=>__('Header', self::$plugin_id), self::LOC_FOOTER=>__('Footer', self::$plugin_id), self::LOC_WIDGET=>__('Widget', self::$plugin_id), self::LOC_B_PAGE_CONTENT=>__('Before page content', self::$plugin_id), self::LOC_A_PAGE_CONTENT=>__('After page content', self::$plugin_id), self::LOC_B_POST_CONTENT=>__('Before post content', self::$plugin_id), self::LOC_A_POST_EXCERPT=>__('After post excerpt', self::$plugin_id), self::LOC_A_POST_CONTENT=>__('After post content', self::$plugin_id), self::LOC_MANUAL=>__('Manual', self::$plugin_id), self::LOC_SHORTCODE=>__('Shortcode', self::$plugin_id));
		$this->location->input = new q2w3_select_input(array('name'=>$this->input_name($this->location->col_name), 'value_to_select'=>$this->location->input_values, 'class'=>'select_input', 'id'=>'select_location'));
		$this->location->help = __('Location where to place included code. If Widget selected, go to the Widget admin page to setup widget position. In Manual mode you must insert plugin function directly in to theme file', self::$plugin_id);
		$this->location->conv = new q2w3_select_conv($this->location, self::$plugin_id);
		$this->location->table_view = true;
		$this->location->table_view_change = true;
		$this->location->single_view = true;
		
		$this->widget_title->name = __('Widget Title', self::$plugin_id);
		$this->widget_title->col_name = 'widget_title';
		$this->widget_title->col_type = 'VARCHAR(255)';
		$this->widget_title->input = new q2w3_text_input(array('name'=>$this->input_name($this->widget_title->col_name), 'maxlength'=>255));
		$this->widget_title->conv = new q2w3_text_conv();
		$this->widget_title->table_view = false;
		$this->widget_title->table_view_change = false;
		$this->widget_title->single_view = false;
		
		$this->priority->name = __('Priority', self::$plugin_id);
		$this->priority->col_name = 'priority';
		$this->priority->col_type = 'INT';
		$this->priority->input = new q2w3_text_input(array('name'=>$this->input_name($this->priority->col_name), 'class'=>'digit_input', 'maxlenght'=>4));
		$this->priority->help = __('Determines display order of includes in the same location', self::$plugin_id);
		$this->priority->conv = new q2w3_priority_conv();
		$this->priority->table_view = true;
		$this->priority->table_view_change = true;
		$this->priority->single_view = true;
				
		$this->inc_pages->name = __('Insert on pages', self::$plugin_id);
		$this->inc_pages->col_name = 'inc_pages';
		$this->inc_pages->col_type = 'TEXT NOT NULL';
		$this->inc_pages->input = new q2w3_wp_page_select_input(array('name'=>$this->input_name($this->inc_pages->col_name), 'id'=>'inc_pages_select', 'type'=>'include')); 
		$this->inc_pages->help = __('Pages where code CAN BE shown. Hold CTRL key for multiple select or deselect pages. Required field', self::$plugin_id);
		$this->inc_pages->conv = new q2w3_select_page_conv($this->inc_pages, self::$plugin_id);
		$this->inc_pages->table_view = false;
		$this->inc_pages->table_view_change = true;
		$this->inc_pages->single_view = true;
		
		$this->exc_pages->name = __('Exclude pages', self::$plugin_id);
		$this->exc_pages->col_name = 'exc_pages';
		$this->exc_pages->col_type = 'TEXT';
		$this->exc_pages->input = new q2w3_wp_page_select_input(array('name'=>$this->input_name($this->exc_pages->col_name), 'id'=>'exc_pages_select', 'type'=>'exclude')); 
		$this->exc_pages->help = __('Pages where code CAN NOT be shown. Hold CTRL key for multiple select or deselect pages', self::$plugin_id);
		$this->exc_pages->conv = new q2w3_select_page_conv($this->exc_pages, self::$plugin_id);
		$this->exc_pages->table_view = false;
		$this->exc_pages->table_view_change = true;
		$this->exc_pages->single_view = true;
		
		$this->hide_from->name = __('Hide from user', self::$plugin_id);
		$this->hide_from->col_name = 'hide_from_admin';
		$this->hide_from->col_type = 'TEXT';
		$this->hide_from->input = new q2w3_checkbox_multi_input(array('name'=>$this->input_name($this->hide_from->col_name), 'value_to_select'=>q2w3_table_func::wp_user_roles()));
		$this->hide_from->help = __('Code will be hidden for all users with the selected roles. Visitor - is a virtual user role for all not registered visitors of your site. Use example: if you select Visitor role only, code will be shown only for registered users.', self::$plugin_id);
		$this->hide_from->conv = new q2w3_checkbox_multi_conv('', self::$plugin_id, q2w3_table_func::wp_user_roles());
		$this->hide_from->table_view = true;
		$this->hide_from->table_view_change = true;
		$this->hide_from->single_view = true;

		$this->code_align->name = __('Align', self::$plugin_id);
		$this->code_align->col_name = 'code_align';
		$this->code_align->col_type = 'INT';
		$this->code_align->input_values = array(''=>__('Align Not Set', self::$plugin_id), self::ALIGN_LEFT=>__('Align Left', self::$plugin_id), self::ALIGN_CENTER=>__('Align Center', self::$plugin_id), self::ALIGN_RIGHT=>__('Align Right', self::$plugin_id));
		$this->code_align->input = new q2w3_select_input(array('name'=>$this->input_name($this->code_align->col_name), 'value_to_select'=>$this->code_align->input_values, 'class'=>'select_input'));
		$this->code_align->conv = new q2w3_text_conv($this->code_align, self::$plugin_id);
		$this->code_align->table_view = false;
		$this->code_align->table_view_change = false;
		$this->code_align->single_view = false;
		
		$this->code->name = __('Include code', self::$plugin_id);
		$this->code->col_name = 'code';
		$this->code->col_type = 'TEXT NOT NULL';
		$this->code->input = new q2w3_textarea_input(array('name'=>$this->input_name($this->code->col_name), 'class'=>'textarea_input', 'id'=>'inc_code'));
		$this->code->help = __('Included code. Required field. Any type of HTML, CSS, JAVASCRIPT and PHP code. PHP code must begins with php open tag. Align option allows you to wrap code in a div with selected text-align css propertie. Useful for aligning Google AdSense blocks and so on', self::$plugin_id);
		$this->code->table_view = false;
		$this->code->table_view_change = false;
		$this->code->single_view = true;
		
	}

	/**
 	 * Here user input is checked 
 	 * 
 	 */
	protected function check_input_data() {
	
		if (!$this->title->val) {
		
			new q2w3_table_sys_msg(__('You must input Description field!', self::$plugin_id));
			
			return false;
		
		} elseif (!intval($this->priority->val)) {
		
			new q2w3_table_sys_msg(__('Priority number must be integer!', self::$plugin_id));
			
			return false;
				
		} elseif (!$this->inc_pages->val) {
		
			new q2w3_table_sys_msg(__('You must select at least one page where to place included code!', self::$plugin_id));
			
			return false;
				
		} elseif (!$this->code->val) {
		
			new q2w3_table_sys_msg(__('You must enter included code!', self::$plugin_id));
			
			return false;
			
		}
		
		return true;
	
	}
	
	/**
 	 * Returns array of special page selectors 
 	 * 
 	 */
	public static function page_selectors($plugin_id = false) {
		
		if (!$plugin_id) $plugin_id = self::$plugin_id;
		
		return array(
			self::ALL => __('All', $plugin_id), 
			self::FRONT_PAGE => __('Front page', $plugin_id), 
			self::POST_TYPES_PAGES => __('All Post Types pages', $plugin_id),
			self::TAX_PAGES => __('Taxonomies pages', $plugin_id), 
			self::DATE_PAGES => __('Date pages', $plugin_id),
			self::AUTHOR_PAGES => __('Author pages', $plugin_id), 
			self::ATTACHMENT_PAGES => __('Attachment pages', $plugin_id), 
			self::PAGED_PAGES => __('Paged pages', $plugin_id), 
			self::SEARCH_PAGE => __('Search page', $plugin_id), 
			self::PREVIEW_PAGE => __('Preview page', $plugin_id), 
			self::PAGE_404 => __('Page 404', $plugin_id)
		);
				
	}
	
	/**
 	 * Changes status of the include 
 	 * 
 	 */
	public function change_status($status) {
	
		global $wpdb;
		
		if ($this->id->col_name && $this->id->val && $this->status->col_name && $status) {
			
			if ($wpdb->update($this->table(), array($this->status->col_name=>$status), array($this->id->col_name=>$this->id->val)) !== false) {
								
				new q2w3_table_sys_msg(__('Status changed', self::$plugin_id));
				
			}
			
		}
			
	}
	
	/**
 	 * Returns javascript needed for include edit form 
 	 * 
 	 */
	public function js() {
		
		$this->widget_title->input->value = $this->widget_title->val;
		
		$this->code_align->input->value = $this->code_align->val;
		
		$zcbpath = q2w3_table_func::folder_url() .'/_js/zeroclipboard/';
		
		$zcbptext = __('Copy to clipboard', self::$plugin_id);
		
		$js = '<script src="'. $zcbpath .'ZeroClipboard.js"></script>
<script type="text/javascript">
ZeroClipboard.setMoviePath(\''. $zcbpath .'ZeroClipboard.swf\');


jQuery("#select_location").parent("td").append(\'<span id="manual_include_code"></span>\');
jQuery("#inc_code").parent("td").prepend(\'<span id="code_align"></span>\');
jQuery("#select_location").change(function() {
	var location = jQuery(this).attr("value");
	if (location == '. self::LOC_WIDGET .' || location == '. self::LOC_MANUAL .') {
		jQuery(this).parent("td").parent("tr").next("tr").hide();	
	} else {
		jQuery(this).parent("td").parent("tr").next("tr").show();
	}	
	if (location == '. self::LOC_MANUAL .') {
		jQuery("#manual_include_code").text("");
		jQuery("#manual_include_code").append("<span id=\"text_to_copy\">&lt;?php if ( method_exists(\'q2w3_inc_manager\', \'manual_inc\') ) q2w3_inc_manager::manual_inc('. $_GET['id'] .'); ?&gt;</span> <input type=\"button\" value=\"'. $zcbptext .'\" id=\"copy\" class=\"button-secondary\" />");
		zerocopy();
	} else if (location == '. self::LOC_WIDGET .') {
		jQuery("#manual_include_code").text("");
		jQuery("#manual_include_code").append(\''. $this->widget_title->name .': '. trim($this->widget_title->input) .'\');
	} else if (location == '. self::LOC_SHORTCODE .') {
		jQuery("#manual_include_code").text("");
		jQuery("#manual_include_code").append("<span id=\"text_to_copy\">[include id=&quot;'. $_GET['id'] .'&quot; title=&quot;" + get_inc_title() + "&quot;]</span> <input type=\"button\" value=\"'. $zcbptext .'\" id=\"copy\" class=\"button-secondary\" />");
		zerocopy();
	} else {
		jQuery("#manual_include_code").text("");
	}
	if (location != '. self::LOC_HEADER .') {
		jQuery("#code_align").text("");
		jQuery("#code_align").append(\''. str_replace(PHP_EOL,'',$this->code_align->input) .'\');
	} else {
		jQuery("#code_align").text("");
	}
}).trigger("change");

function get_inc_title() {
	return jQuery("input[name=\'propertie[description]\']").attr("value");
}

function zerocopy() {
	var clip = new ZeroClipboard.Client();
	clip.setText(\'\');
	clip.addEventListener(\'mouseDown\', function(){
 		var pre = document.getElementById(\'text_to_copy\');
 		clip.setText(htmlDecode(pre.innerHTML));
	});
	clip.glue(\'copy\');
}

function htmlDecode(value){ 
 	return jQuery(\'<div/>\').html(value).text(); 
}
</script>';

		return $js;

	}
			
}



/**
 * @author Max Bond
 * 
 * Q2W3 Table system messages class
 * 
 * Uses sessions
 *
 */
class q2w3_table_sys_msg {
	
	
	
	public function __construct($msg) {
		
		if (!session_id()) @session_start(); // This will generate warning: 'Cannot send session cookie - headers already sent by' 

		if ($msg) $_SESSION['q2w3_table_sys_msg'][] = $msg; 
		
	}
	
	public static function display() {
	
		if (!session_id()) @session_start(); 
	
		if (key_exists('q2w3_table_sys_msg', $_SESSION) && is_array($_SESSION['q2w3_table_sys_msg'])) {
			
			$msgs = array_unique($_SESSION['q2w3_table_sys_msg']);

			unset($_SESSION['q2w3_table_sys_msg']);
			
		}  else {
			
			return false;
		
		}
			
		foreach ($msgs as $msg) {
			
			$res .= '<div class="updated fade"><p>'. $msg .'</p></div>'.PHP_EOL; // Standard WP sys msg style
			
		}
		
		return $res; 
		
	}
	
}


?>